local L = AceLibrary("AceLocale-2.2"):new("ShammySpy")
local deformat = AceLibrary("Deformat-2.0")

--[[ Ace Events ----------------------------------------------------------------------]]
function ShammySpy:OnInitialize()
	local _, englishClass = UnitClass("player")
	if (englishClass == "SHAMAN") then
		
	end
end

function ShammySpy:OnEnable()
	local _, englishClass = UnitClass("player")
	if (englishClass == "SHAMAN") then
		-- Setup the frame
		self:SetupFrames()
		self:UpdateLock()
		
		-- Register the WoW events
		self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
		self:RegisterEvent("BAG_UPDATE");
		self:RegisterEvent("PLAYER_DEAD");
		self:RegisterEvent("CHAT_MSG_COMBAT_FRIENDLY_DEATH");
		
		-- Queue a check in case Reincarnation is on CD
		if ( not (self:IsEventScheduled("UpdateFrames")) ) then
			self:ScheduleRepeatingEvent("UpdateFrames", self.UpdateFrames, 1, self)
		end
	end
end

function ShammySpy:OnDisable()

end

--[[ Functions -----------------------------------------------------------------------]]
function ShammySpy:FormatTime(seconds)
	seconds = floor(seconds);
	if (seconds > 60) then
		secs = mod(seconds, 60);
		mins = (seconds - secs) / 60;
		return mins..":"..string.sub("00"..secs, -2)
	else
		return "00:"..string.sub("00"..seconds, -2);
	end
end

function ShammySpy:KillTotemElement(element)
	local frame = getglobal("ShammySpy"..element.."Frame");
	local text = getglobal("ShammySpy"..element.."FrameText1");
	
	frame.Warn = nil;
	frame.Die = nil;
	frame.spell = nil;
	frame:SetBackdrop({
		bgFile = "Interface/Icons/Spell_Totem_WardOfDraining",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		tile = false, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right =5, top = 5, bottom = 5 }
	})
	frame:SetBackdropColor(1,1,1,(1/3))
	text:SetText(L[element])
end

function ShammySpy:CountAnkhs()
	local ankhCount = 0;
	for bagId=0, 4, 1 do
		for slotId=1, GetContainerNumSlots(bagId), 1 do
			local itemLink = GetContainerItemLink(bagId, slotId);
			if (itemLink) then
				local _, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.+%]");
				if (itemString) then
					local _, _, itemId = string.find(itemString, "^item:(%d+):(%d+):(%d+):(%d+):(%d+):(%d+):(%-?%d+):(%-?%d+)");
					if (itemId) then
						if ( (itemId == 17030) or (itemId == "17030") ) then
							local _, itemCount = GetContainerItemInfo(bagId, slotId);
							ankhCount = ankhCount + itemCount;
						end
					end
				end
			end
		end
	end
	
	local text = getglobal("ShammySpyReincarnationFrameText2");
	text:SetText(ankhCount)
end

function ShammySpy:GetSpellID(spellNameFind)
	local spellCheckID = 1;
	local spellID = nil;
	while true do
		local spellName, spellRank = GetSpellName(spellCheckID, BOOKTYPE_SPELL)
		
		if (spellName == spellNameFind) then spellID = spellCheckID; do break end end
   	if not spellName then do break end end
		
		spellCheckID = spellCheckID + 1
	end
	
	return spellID;
end

function ShammySpy:GetReincarnationID()
	if (not self.reincarnationID) then
		self.reincarnationID = ShammySpy:GetSpellID(L["Reincarnation"])
	else
		local spellName, spellRank = GetSpellName(self.reincarnationID, BOOKTYPE_SPELL)
		if (spellName ~= L["Reincarnation"]) then
			self.reincarnationID = ShammySpy:GetSpellID(L["Reincarnation"])
		end
	end
end

function ShammySpy:DelayCheck()
	if ( not (self:IsEventScheduled("UpdateFrames")) ) then
		self:ScheduleRepeatingEvent("UpdateFrames", self.UpdateFrames, 1, self)
	end
end

--[[ Custom events -------------------------------------------------------------------]]
function ShammySpy:UpdateFrames()
	local canStop = true;
	
	for _,element in pairs({"Earth","Fire","Water","Air"}) do
		local frame = getglobal("ShammySpy"..element.."Frame");
		local text = getglobal("ShammySpy"..element.."FrameText1");
		
		if (frame.Warn) then
			if (time() >= frame.Warn) then
				UIErrorsFrame:AddMessage(frame.spell.." ("..L[element]..") is about to expire...", 1.0, 1.0, 0.0, 1.0, 3);
				frame.Warn = nil;
			end
		elseif (frame.Die) then
			if (time() >= frame.Die) then
				UIErrorsFrame:AddMessage(frame.spell.." ("..L[element]..") has expired.", 1.0, 0.5, 0.0, 1.0, 5);
				frame.Die = nil;
				
				self:KillTotemElement(element)
			end
		end
		
		if (frame.Die) then
			text:SetText(self:FormatTime(frame.Die - time()))
			canStop = false;
		end
	end
	
	--[[ Check Reincarnation Cooldown ]]
	self:GetReincarnationID();
	local start, duration = GetSpellCooldown(self.reincarnationID, BOOKTYPE_SPELL);
	if ( start > 0 and duration > 0) then
		local frame = getglobal("ShammySpyReincarnationFrame");
		local text = getglobal("ShammySpyReincarnationFrameText1");
		local reincarnationCD = duration - ( GetTime() - start);
		
		frame:SetBackdrop({
			bgFile = "Interface/Icons/Spell_Nature_Reincarnation",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = false, tileSize = 16, edgeSize = 16,
			insets = { left = 5, right =5, top = 5, bottom = 5 }
		})
		frame:SetBackdropColor(1,1,1,1)
		text:SetText(self:FormatTime(reincarnationCD))
		canStop = false;
	else
		local frame = getglobal("ShammySpyReincarnationFrame");
		local text = getglobal("ShammySpyReincarnationFrameText1");
		
		frame:SetBackdrop({
			bgFile = "Interface/Icons/Spell_Nature_Reincarnation",
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = false, tileSize = 16, edgeSize = 16,
			insets = { left = 5, right =5, top = 5, bottom = 5 }
		})
		frame:SetBackdropColor(1,1,1,(1/3))
		text:SetText("00:00")
	end
	
	if (canStop and self:IsEventScheduled("UpdateFrames")) then
		self:CancelScheduledEvent("UpdateFrames")
	end
end

--[[ Subscribed events ---------------------------------------------------------------]]
function ShammySpy:UNIT_SPELLCAST_SUCCEEDED(unit, spell, rank) -- "player", spell, rank
	if unit ~= "player" then return end
	
	if (ShammySpy.Totems[spell]) then
		local frame = getglobal("ShammySpy"..ShammySpy.Totems[spell].Type.."Frame");
		local text = getglobal("ShammySpy"..ShammySpy.Totems[spell].Type.."FrameText1")
		
		local TotemTime = ShammySpy.Totems[spell].Time;
		if ( (TotemTime == 0) or (TotemTime == "0") ) then
			if (ShammySpy.Totems[spell][rank]) then
				TotemTime = ShammySpy.Totems[spell][rank].Time
			end
		end
		
		frame:SetBackdrop({
			bgFile = "Interface/Icons/"..ShammySpy.Totems[spell].Icon,
			edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
			tile = false, tileSize = 16, edgeSize = 16,
			insets = { left = 5, right =5, top = 5, bottom = 5 }
		})
		frame:SetBackdropColor(1,1,1,1)
		if (TotemTime > 10) then frame.Warn = time() + TotemTime - 10; end
		frame.Die = time() + TotemTime;
		frame.spell = spell;
		text:SetText(self:FormatTime(frame.Die - time()))
		
		if ( not (self:IsEventScheduled("UpdateFrames")) ) then
			self:ScheduleRepeatingEvent("UpdateFrames", self.UpdateFrames, 1, self)
		end
	elseif (spell == L["Totemic Call"]) then
		-- kill all totem timers
		for _,element in pairs({"Earth","Fire","Water","Air"}) do
			local frame = getglobal("ShammySpy"..element.."Frame");
			frame.Warn = nil;
			frame.Die = nil;
			self:KillTotemElement(element)
		end
	elseif (spell == L["Reincarnation"]) then
		if ( not (self:IsEventScheduled("UpdateFrames")) ) then
			self:ScheduleRepeatingEvent("UpdateFrames", self.UpdateFrames, 1, self)
		end
		self:ScheduleRepeatingEvent("DelayCheck", self.DelayCheck, 7, self)
	end
end

function ShammySpy:PLAYER_DEAD()
	for _,element in pairs({"Earth","Fire","Water","Air"}) do
		local frame = getglobal("ShammySpy"..element.."Frame");
		frame.Warn = nil;
		frame.Die = nil;
		self:KillTotemElement(element)
	end
end

function ShammySpy:BAG_UPDATE()
	self:CountAnkhs();
end

function ShammySpy:CHAT_MSG_COMBAT_FRIENDLY_DEATH(message)
	local item = deformat(message, UNITDESTROYEDOTHER)
	if (item) then -- %s is destroyed.
		-- Check for time left and ignore if 0-2 sec
		-- self:Print(item); -- FUCK has roman numbers for rank :/
		
		for _,element in pairs({"Earth","Fire","Water","Air"}) do
			local frame = getglobal("ShammySpy"..element.."Frame");
			if (frame.Die) then
				local _, _, theTotem = string.find(item, "^("..frame.spell..")");
				if ( (theTotem) and (time() <= (frame.Die - 1)) ) then
					UIErrorsFrame:AddMessage(frame.spell.." ("..L[element]..") ended prematurely!!!", 1.0, 0.0, 0.0, 1.0, 3);
					self:KillTotemElement(element)
				end
			end
		end
		
	end
end

--[[
	Objects:
		ShammySpyEarthFrame | ShammySpyEarthFrameText1
		ShammySpyFireFrame | ShammySpyFireFrameText1
		ShammySpyWaterFrame | ShammySpyWaterFrameText1
		ShammySpyAirFrame | ShammySpyAirFrameText1
		ShammySpyReincarnationFrame | ShammySpyReincarnationFrameText1 | ShammySpyReincarnationFrameText2
		
		/script getglobal("ShammySpyReincarnationFrameText2"):SetText("1");
		/script getglobal("ShammySpyReincarnationFrameText"):SetText("1");
		/script ShammySpyReincarnationFrameText1:SetText("7");
]]
